@@ -21,7 +21,7 @@ import Text.CSL.Proc
 import Text.CSL.Output.Pandoc (renderPandoc, renderPandoc')
 import qualified Text.CSL.Style as CSL
 import Text.CSL.Parser
-import Text.CSL.Output.Pandoc ( headInline, tailFirstInlineStr, initInline,
+import Text.CSL.Output.Pandoc ( headInline, tailInline, initInline,
                                 toCapital )
 import Text.CSL.Data (getDefaultCSL)
 import Text.Parsec hiding (State, (<|>))
@@ -33,6 +33,7 @@ import System.Directory (doesFileExist, getAppUserDataDirectory)
 import Text.CSL.Util (findFile, splitStrWhen, tr', parseRomanNumeral, trim)
 import System.IO.Error (isDoesNotExistError)
 import Data.Maybe (fromMaybe)
+import Text.XML (XMLException(..))
 
 -- | Process a 'Pandoc' document by adding citations formatted
 -- according to a CSL style.  Add a bibliography (if one is called
@@ -167,7 +168,9 @@ processCites' (Pandoc meta blocks) = do
                                  else getDefaultCSL
                             Nothing -> getDefaultCSL
   csl <- case cslfile of
-               Just f | not (null f) -> readCSLFile mbLocale f
+               Just f | not (null f) -> E.catch
+                   (readCSLFile mbLocale f) $ \e ->
+                        E.throwIO (InvalidXMLFile f e)
                _ -> do
                  -- get default CSL: look first in ~/.csl, and take
                  -- from distribution if not found
@@ -269,7 +272,7 @@ mvPunct :: Bool -> Style -> [Inline] -> [Inline]
 mvPunct _ _ (x : Space : xs) | isSpacy x = x : xs
 mvPunct moveNotes _ (s : x : ys) | isSpacy s, isNote x, startWithPunct ys =
   if moveNotes
-     then Str (headInline ys) : x : tailFirstInlineStr ys
+     then Str (headInline ys) : x : tailInline ys
      else x : ys
 mvPunct moveNotes _ (Cite cs ils : ys) |
      length ils > 1
@@ -277,7 +280,7 @@ mvPunct moveNotes _ (Cite cs ils : ys) |
    , startWithPunct ys
    , moveNotes
    = Cite cs (init ils ++ [Str (headInline ys) | not (endWithPunct False (init ils))]
-     ++ [last ils]) : tailFirstInlineStr ys
+     ++ [last ils]) : tailInline ys
 mvPunct moveNotes sty (q@(Quoted _ _) : w@(Str _) : x : ys)
   | isNote x, isPunctuationInQuote sty, moveNotes  =
     mvPunctInsideQuote q w ++ (x : ys)
@@ -302,8 +305,8 @@ startWithPunct = and . map (`elem` (".,;:!?" :: String)) . headInline
 
 deNote :: Pandoc -> Pandoc
 deNote = topDown go
-  where go (Cite (c:cs) [Note xs]) =
-            Cite (c:cs) [Note $ sanitize xs]
+  where go (Cite (c:cs) [Note [Para xs]]) =
+            Cite (c:cs) [Note [Para $ toCapital xs]]
         go (Note xs) = Note $ topDown go' xs
         go x = x
         go' (x : Cite cs [Note [Para xs]] : ys) | not (isSpacy x) =
@@ -313,18 +316,16 @@ deNote = topDown go
         go' (Cite cs [Note [Para xs]] : ys) = comb (\zs -> [Cite cs zs]) xs ys
         go' (Note [Para xs] : ys) = comb id xs ys
         go' xs = xs
-        removeLeadingPunct (Str [c] : s : xs)
-          | isSpacy s && (c == ',' || c == '.' || c == ':') = xs
-        removeLeadingPunct xs = xs
-        comb f xs ys =
-           let xs' = if startWithPunct ys && endWithPunct True xs
-                        then initInline $ removeLeadingPunct xs
-                        else removeLeadingPunct xs
-           in f xs' ++ ys
-        sanitize :: [Block] -> [Block]
-        sanitize [Para xs] =
-           [Para $ toCapital xs ++ if endWithPunct False xs then [Space] else []]
-        sanitize bs = bs
+
+comb :: ([Inline] -> [Inline]) -> [Inline] -> [Inline] -> [Inline]
+comb f xs ys =
+  let xs' = if startWithPunct ys && endWithPunct True xs
+               then initInline $ removeLeadingPunct xs
+               else removeLeadingPunct xs
+      removeLeadingPunct (Str [c] : s : zs)
+          | isSpacy s && (c == ',' || c == '.' || c == ':') = zs
+      removeLeadingPunct zs = zs
+  in  f xs' ++ ys
 
 -- | Retrieve all citations from a 'Pandoc' docuument. To be used with
 -- 'query'.
@@ -368,7 +369,7 @@ toCslCite locMap c
 locatorWords :: LocatorMap -> [Inline] -> (String, String, [Inline])
 locatorWords locMap inp =
   case parse (pLocatorWords locMap) "suffix" $
-         splitStrWhen (\c -> isPunctuation c || isSpace c) inp of
+         splitStrWhen (\c -> isLocatorPunct c || isSpace c) inp of
        Right r   -> r
        Left _    -> ("","",inp)
 
@@ -434,11 +435,13 @@ pDigit = do
       _ -> mzero
 
 pLocatorPunct :: Parsec [Inline] st Inline
-pLocatorPunct = pMatch isLocatorPunct
+pLocatorPunct = pMatch isLocatorPunct'
+  where isLocatorPunct' (Str [c]) = isLocatorPunct c
+        isLocatorPunct' _ = False
 
-isLocatorPunct :: Inline -> Bool
-isLocatorPunct (Str [c]) = isPunctuation c
-isLocatorPunct _         = False
+isLocatorPunct :: Char -> Bool
+isLocatorPunct ':' = False
+isLocatorPunct c   = isPunctuation c
 
 type LocatorMap = M.Map String String
 
