@@ -1,7 +1,13 @@
 {-# LANGUAGE GeneralizedNewtypeDeriving, PatternGuards, OverloadedStrings,
   DeriveDataTypeable, ExistentialQuantification, FlexibleInstances,
   ScopedTypeVariables, GeneralizedNewtypeDeriving, IncoherentInstances,
-  DeriveGeneric #-}
+  DeriveGeneric, CPP #-}
+#if MIN_VERSION_base(4,8,0)
+#define OVERLAPS {-# OVERLAPPING #-}
+#else
+{-# LANGUAGE OverlappingInstances #-}
+#define OVERLAPS
+#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Text.CSL.Reference
@@ -161,7 +167,8 @@ instance ToYaml RefDate where
                       , "circa" &= T.pack (if circa r then "1" else "")
                       ]
 
-instance FromJSON [RefDate] where
+instance OVERLAPS
+         FromJSON [RefDate] where
   parseJSON (Array xs) = mapM parseJSON $ V.toList xs
   parseJSON (Object v) = do
     dateParts <- v .:? "date-parts"
@@ -196,7 +203,8 @@ toDatePart refdate =
          (Just y, Nothing, Nothing)  -> [y]
          _                           -> []
 
-instance ToJSON [RefDate] where
+instance OVERLAPS
+         ToJSON [RefDate] where
   toJSON [] = Array V.empty
   toJSON xs = object' $
     case filter (not . null) (map toDatePart xs) of
@@ -205,9 +213,6 @@ instance ToJSON [RefDate] where
                  ["circa" .= (1 :: Int) | or (map circa xs)] ++
                  ["season" .= s | s <- map season xs, s /= mempty])
 
--- instance ToJSON [RefDate]
--- toJSON xs  = Array (V.fromList $ map toJSON xs)
-
 setCirca :: Bool -> RefDate -> RefDate
 setCirca circa' rd = rd{ circa = circa' }
 
@@ -379,7 +384,7 @@ data Reference =
 
 instance FromJSON Reference where
   parseJSON (Object v') = do
-     v <- parseSuppFields v'
+     v <- parseSuppFields v' <|> return v'
      addPageFirst <$> (Reference <$>
        v .:? "id" .!= "" <*>
        v .:? "type" .!= NoType <*>
@@ -472,7 +477,7 @@ instance FromJSON Reference where
 -- see http://gsl-nagoya-u.net/http/pub/citeproc-doc.html#supplementary-fields
 parseSuppFields :: Aeson.Object -> Parser Aeson.Object
 parseSuppFields o = do
-  nt <- o .: "note" <|> return ("" :: String)
+  nt <- o .: "note"
   case P.parse noteFields "note" nt of
        Left err -> fail (show err)
        Right fs -> return $ foldr (\(k,v) x -> H.insert k v x) o fs
