@@ -106,9 +106,8 @@ import Text.CSL.Util (mb, parseBool, parseString, (.#?), (.#:), query,
 import Data.Yaml.Builder(ToYaml(..))
 import qualified Data.Yaml.Builder as Y
 import Text.Pandoc.Definition hiding (Citation, Cite)
-import Text.Pandoc (readHtml, writeMarkdown,
-                    WriterOptions(..), ReaderOptions(..), WrapOption(..),
-                    bottomUp, def)
+import Text.Pandoc (bottomUp)
+import Text.CSL.Compat.Pandoc (readHtml, writeMarkdown)
 import qualified Text.Pandoc.Walk as Walk
 import qualified Text.Pandoc.Builder as B
 import qualified Data.Text as T
@@ -130,19 +129,10 @@ import qualified Data.Vector as V
 
 readCSLString :: String -> [Inline]
 readCSLString s = Walk.walk handleSmallCapsSpans
-                $ case readHtml def{ readerSmart = True
-                                   , readerParseRaw = True }
-                                (adjustScTags s) of
-#if MIN_VERSION_pandoc(1,14,0)
-                        Right (Pandoc _ [Plain ils])   -> ils
-                        Right (Pandoc _ [Para  ils])   -> ils
-                        Right (Pandoc _ x)             -> Walk.query (:[]) x
-                        Left  _                        -> []
-#else
+                $ case readHtml (adjustScTags s) of
                         Pandoc _ [Plain ils]   -> ils
                         Pandoc _ [Para  ils]   -> ils
                         Pandoc _ x             -> Walk.query (:[]) x
-#endif
   -- this is needed for versions of pandoc that don't turn
   -- a span with font-variant:small-caps into a SmallCaps element:
   where handleSmallCapsSpans (Span ("",[],[("style",sty)]) ils)
@@ -164,13 +154,13 @@ adjustScTags zs =
 
 writeYAMLString :: [Inline] -> String
 writeYAMLString ils =
-  trimr $ writeMarkdown def{writerWrapText = WrapNone}
+  trimr $ writeMarkdown
         $ Pandoc nullMeta
           [Plain $ bottomUp (concatMap (adjustCSL False)) ils]
 
 writeCSLString :: [Inline] -> String
 writeCSLString ils =
-  trimr $ writeMarkdown def{writerWrapText = WrapNone}
+  trimr $ writeMarkdown
         $ Pandoc nullMeta
           [Plain $ bottomUp (concatMap (adjustCSL True)) ils]
 
