@@ -1,6 +1,5 @@
+{-# LANGUAGE CPP #-}
 module Main where
-import JSON
-import System.Process
 import System.Exit
 import System.Directory
 import System.FilePath
@@ -9,18 +8,10 @@ import System.IO
 import Data.Monoid (mempty)
 import System.IO.Temp (withSystemTempDirectory)
 import System.Process (rawSystem)
-import Text.Printf
 import qualified Data.Aeson as Aeson
 import Text.Pandoc.Definition
-import qualified Data.ByteString.Lazy.Char8 as BL
-import qualified Data.ByteString.Char8 as B
 import qualified Text.Pandoc.UTF8 as UTF8
-import Text.Pandoc.Shared (normalize)
-import Text.Pandoc.Process (pipeProcess)
-import Text.Pandoc.Options (WriterOptions(..))
-import qualified Data.Yaml as Yaml
-import Text.Pandoc (writeNative, writeHtmlString, readNative, def)
-import Text.CSL.Pandoc (processCites')
+import Text.CSL.Compat.Pandoc (writeNative, pipeProcess)
 import Data.List (isSuffixOf)
 import System.Environment
 import Control.Monad (when)
@@ -64,30 +55,33 @@ testCase regenerate csl = do
   indataNative <- UTF8.readFile $ "tests/" ++ csl ++ ".in.native"
   expectedNative <- UTF8.readFile $ "tests/" ++ csl ++ ".expected.native"
   let jsonIn = Aeson.encode $ (read indataNative :: Pandoc)
-  let expectedDoc = normalize $ read expectedNative
+  let expectedDoc = read expectedNative
   testProgPath <- getExecutablePath
   let pandocCiteprocPath = takeDirectory testProgPath </> ".." </>
         "pandoc-citeproc" </> "pandoc-citeproc"
-  (ec, jsonOut, errout) <- pipeProcess
+  (ec, jsonOut) <- pipeProcess
                      (Just [("LANG","en_US.UTF-8"),("HOME",".")])
                      pandocCiteprocPath
                      [] jsonIn
   if ec == ExitSuccess
      then do
-       let outDoc = normalize $ fromMaybe mempty $ Aeson.decode $ jsonOut
+       let outDoc = fromMaybe mempty $ Aeson.decode $ jsonOut
        if outDoc == expectedDoc
           then err "PASSED" >> return Passed
           else do
              err $ "FAILED"
-             showDiff (writeNative def expectedDoc) (writeNative def outDoc)
+             showDiff (writeNative expectedDoc) (writeNative outDoc)
              when regenerate $
                UTF8.writeFile ("tests/" ++ csl ++ ".expected.native") $
-                  writeNative def{ writerStandalone = True } outDoc
+#if MIN_VERSION_pandoc(1,19,0)
+                  writeNative outDoc
+#else
+                  writeNative outDoc
+#endif
              return Failed
      else do
        err "ERROR"
        err $ "Error status " ++ show ec
-       err $ UTF8.toStringLazy errout
        return Errored
 
 showDiff :: String -> String -> IO ()
@@ -114,7 +108,7 @@ biblio2yamlTest fp = do
   testProgPath <- getExecutablePath
   let pandocCiteprocPath = takeDirectory testProgPath </> ".." </>
         "pandoc-citeproc" </> "pandoc-citeproc"
-  (ec, result', errout) <- pipeProcess
+  (ec, result') <- pipeProcess
                      (Just [("LANG","en_US.UTF-8"),("HOME",".")])
                      pandocCiteprocPath
                      ["--bib2yaml", "-f", drop 1 $ takeExtension fp]
@@ -131,5 +125,4 @@ biblio2yamlTest fp = do
      else do
        err "ERROR"
        err $ "Error status " ++ show ec
-       err $ UTF8.toStringLazy errout
        return Errored
